#--------------------------------------------------------------------------
#     This file is part of BKchem - a chemical drawing program
#     Copyright (C) 2002  Beda Kosata <kosatab@vscht.cz>

#     This program is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation; either version 2 of the License, or
#     (at your option) any later version.

#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.

#     Complete text of GNU GPL can be found in the file gpl.txt in the
#     main directory of the program

#--------------------------------------------------------------------------
#
# Last edited: $Date: 2002/12/16 19:56:18 $
#
#--------------------------------------------------------------------------

"""Openoffice Draw export plugin"""

import plugin
import xml.dom.minidom as dom
import dom_extensions as dom_ext
import dom_extensions
import math
import operator
import os_support

## DEFINITIONS

class OO_exporter( plugin.exporter):

  def __init__( self, paper):
    self.paper = paper
    self.used_styles = []

  def on_begin( self):
    import tkMessageBox
    yes = tkMessageBox.askyesno( _("Really export?"),
                                 _('This plugin is not finished and will probably not work correctly.') + ' ' +
                                 _('Proceed?'))
    return yes

  def write_to_file( self, name):
    out = dom.Document()
    root = dom_ext.elementUnder( out, 'office:document-content',
                                 (('office:class', 'drawing'),
                                  ('office:version', '1.0'),
                                  ('xmlns:draw', 'http://openoffice.org/2000/drawing'),
                                  ('xmlns:form', "http://openoffice.org/2000/form"),
                                  ('xmlns:office',"http://openoffice.org/2000/office"),
                                  ('xmlns:style',"http://openoffice.org/2000/style"),
                                  ('xmlns:svg',"http://www.w3.org/2000/svg"),
                                  ('xmlns:text',"http://openoffice.org/2000/text"),
                                  ('xmlns:fo',"http://www.w3.org/1999/XSL/Format")))

    self.styles_element = dom_ext.elementUnder( root, 'office:automatic-styles')
##     # drawing page
##     dom_ext.elementUnder( styles, 'style:style', (('style:family', 'drawing-page'),
##                                                   ('style:name', 'dpg')))

    body = dom_ext.elementUnder( root, 'office:body')
    page = dom_ext.elementUnder( body, 'draw:page', (('draw:master-page-name','Default'),
                                                     ('draw:name', 'page1')))
    for o in self.paper.molecules:
      for b in o.bonds:
        self.add_bond( b, page)

    for o in self.paper.molecules:
      for b in o.atoms_map:
        self.add_atom( b, page)

    dom_ext.safe_indent( root)
    
    import tempfile
    fname = tempfile.mktemp()
    f = open( fname, "w")
    f.write( out.toxml())
    f.close()
    import zipfile
    zip = zipfile.ZipFile( name, 'w', zipfile.ZIP_DEFLATED)
    manifest = os_support.get_path( 'Meta-inf/manifest.xml', 'plugin')
    if manifest:
      zip.write( os_support.get_path( 'Meta-inf/manifest.xml', 'plugin'), 'META-INF/manifest.xml')
      zip.write( fname, 'content.xml')
      zip.close()
    else:
      zip.close()
      raise plugin.export_exception( _("The manifest file not found in the plugin directory"))


  def add_bond( self, b, page):
    """adds bond item to page"""
    s = graphics_style( stroke_color='#000000')
    style_name = self.get_appropriate_style_name( s)
    l_group = page
    if b.type <= 3:
      if not (b.type == 2 and b.center): 
        x1, y1, x2, y2 = reduce( operator.add, [o.get_xy() for o in b.get_atoms()])
        x1, y1, x2, y2 = map( lambda a: a*2.54/72, [x1, y1, x2, y2])
        line = dom_extensions.elementUnder( l_group, 'draw:line',
                                            (( 'svg:x1', '%fcm' %  x1),
                                             ( 'svg:y1', '%fcm' %  y1),
                                             ( 'svg:x2', '%fcm' %  x2),
                                             ( 'svg:y2', '%fcm' %  y2),
                                             ( 'draw:layer', 'layout'),
                                             ( 'draw:style-name', style_name)))
      if b.second:
        x1, y1, x2, y2 = self.paper.coords( b.second)
        x1, y1, x2, y2 = map( lambda a: a*2.54/72, [x1, y1, x2, y2])
        line = dom_extensions.elementUnder( l_group, 'draw:line',
                                            (( 'svg:x1', '%fcm' %  x1),
                                             ( 'svg:y1', '%fcm' %  y1),
                                             ( 'svg:x2', '%fcm' %  x2),
                                             ( 'svg:y2', '%fcm' %  y2),
                                             ( 'draw:layer', 'layout'),
                                             ( 'draw:style-name', style_name)))
      if b.third:
        x1, y1, x2, y2 = self.paper.coords( b.third)
        x1, y1, x2, y2 = map( lambda a: a*2.54/72, [x1, y1, x2, y2])
        line = dom_extensions.elementUnder( l_group, 'draw:line',
                                            (( 'svg:x1', '%fcm' %  x1),
                                             ( 'svg:y1', '%fcm' %  y1),
                                             ( 'svg:x2', '%fcm' %  x2),
                                             ( 'svg:y2', '%fcm' %  y2),
                                             ( 'draw:layer', 'layout'),
                                             ( 'draw:style-name', style_name)))
    elif b.type == 4:
      x1, y1, x2, y2, x3, y3 = self.paper.coords( b.item)
      line = dom_extensions.elementUnder( l_group, 'polygon',
                                          (( 'fill', 'black'),
                                           ( 'stroke', 'black'),
                                           ( 'points', '%d %d %d %d %d %d' % (x1, y1, x2, y2, x3, y3))))
    elif b.type == 5:
      for i in b.items:
        x1, y1, x2, y2 = self.paper.coords( i)
        line = dom_extensions.elementUnder( l_group, 'draw:line',
                                            (( 'x1', str( x1)),
                                             ( 'y1', str( y1)),
                                             ( 'x2', str( x2)),
                                             ( 'y2', str( y2))))


  def add_atom( self, a, page):
    """adds atom to document"""
    gr_style = graphics_style()
    gr_style_name = self.get_appropriate_style_name( gr_style)
    para_style = paragraph_style( font_size='%dpt' % a.font_size)
    para_style_name = self.get_appropriate_style_name( para_style)
    txt_style = text_style( font_size='%dpt' % a.font_size)
    txt_style_name = self.get_appropriate_style_name( txt_style)
    if a.show:
      item = a.selector
      x1, y1 = a.get_xy()
      x, y, x2, y2 = self.paper.bbox( item)
      import os
      if os.name == 'nt':
        x += 2  #nasty hack to improve results on win machines (they have larger fonts?!?)
        x2 -= 2 
      x, y, x2, y2 = map( lambda a: a*2.54/72, [x, y, x2, y2])
      box = dom_extensions.elementUnder( page, 'draw:text-box',
                                         (( 'svg:x', '%fcm' %  x),
                                          ( 'svg:y', '%fcm' %  y),
                                          ( 'svg:width', '%fcm' %  (x2-x)),
                                          ( 'svg:height', '%fcm' % (y2-y)),
                                          ( 'draw:style-name', gr_style_name),
                                          ( 'draw:text-style-name', para_style_name)))
      #y1 += (y2-y)/4.0
      text = dom_extensions.elementUnder( box, 'text:p', (('text:style-name', para_style_name),))
      dom_extensions.textOnlyElementUnder( text, 'text:span', a.get_text(), (('text:style-name', txt_style_name),))

  def get_appropriate_style_name( self, style):
    """if same style already exists return its name, otherwise append the current style and return its name"""
    for s in self.used_styles:
      if style == s:
        return s.name
    style.name = style.name + str( len( self.used_styles))
    self.used_styles.append( style)
    self.styles_element.appendChild( style.to_dom())
    return style.name
  
# PLUGIN INTERFACE SPECIFICATION
name = "OpenOffice Draw"
extensions = [".sxd",".zip"]
exporter = OO_exporter


## PRIVATE CLASSES AND FUNCTIONS

class style:

  def __init__( self):
    pass

  def __eq__( self, other):
    for a in self.__dict__:
      if a == 'name':
        continue
      if a in other.__dict__:
        if self.__dict__[a] != other.__dict__[a]:
          return 0
      else:
        return 0
    return 1

  def __ne__( self, other):
    return not self.__eq__( other)

  def to_dom( self):
    pass

class graphics_style( style):

  def __init__( self, name='gr', stroke_color='#ffffff', fill='solid', fill_color='#ffffff', stroke_width=1):
    self.name = name
    self.family = 'graphics'
    self.stroke_color = stroke_color
    self.fill = fill
    self.fill_color = fill_color
    self.stroke_width = stroke_width

  def to_dom( self):
    style = dom.Element( 'style:style')
    dom_extensions.setAttributes( style, (('style:family', self.family),
                                          ('style:name', self.name),
                                          ('style:parent-style-name','standard')))
    dom_extensions.elementUnder( style, 'style:properties', (( 'draw:fill', self.fill),
                                                             ( 'svg:stroke-color', self.stroke_color),
                                                             ( 'draw:fill-color', self.fill_color),
                                                             ( 'svg:stroke-width', '%fcm' % (self.stroke_width * 0.04)),
                                                             ( 'draw:auto-grow-width', 'true'),
                                                             ( 'draw:auto-grow-height', 'true')))
    return style


class paragraph_style( style):

  def __init__( self, name='para', font_size='12pt'):
    self.name = name
    self.family = 'paragraph'
    self.font_size = font_size

  def to_dom( self):
    style = dom.Element( 'style:style')
    dom_extensions.setAttributes( style, (('style:family', self.family),
                                          ('style:name', self.name)))
    dom_extensions.elementUnder( style, 'style:properties', (( 'fo:font-size', self.font_size),
                                                             ( 'fo:font-family', 'Helvetica')))
    return style


class text_style( style):

  def __init__( self, name='text', font_size='12pt'):
    self.name = name
    self.family = 'text'
    self.font_size = font_size

  def to_dom( self):
    style = dom.Element( 'style:style')
    dom_extensions.setAttributes( style, (('style:family', self.family),
                                          ('style:name', self.name)))
    dom_extensions.elementUnder( style, 'style:properties', (( 'fo:font-size', self.font_size),
                                                             ( 'fo:font-family', 'Helvetica')))
    return style

  
